from data.data import Data
from data.data_mod import DataMod
from data.data_obs import DataObs
from data.data_obs import list_season_obs
from data.data_ensembles import DataEns
from data.data_ecearth23 import DataEC23
from data.data_ecmwfS4 import DataS4
from dateutil.relativedelta import *
from data.common import *
import os
import datetime
import copy
import shutil
import sys
from data.set_output_name import *

def calc_seas_percent_nbdays(varini, perlst, explst, year1, year2, monstart, workdir):
    """ this function will calculate and write the output
    of the seasonal (between leadtime month 02 and 04)
    number of day and the percentile per for all years between year1
    and year2 for all exp in explst and all observation in obslist
    -var: the name of the input variable
    -explist: list of experiments or observations
    -year1: first years of the time period
    -year2: last years of the time period
    -monstrt: startmonth
    -workdir: path of the working directory
    """
    
    
    date1=datetime.date(year1,monstart,1)
    date2=datetime.date(year2,monstart,1)

 

    #loop on experiments
    for expname in explst: 
        if (not os.path.isdir(workdir)):
            os.mkdir(workdir)

        workdir=workdir+expname+"/"
        if (not os.path.isdir(workdir)):
            os.mkdir(workdir)
        os.chdir(workdir)
        
        #creating the object according to the type of experiment/observations
        if (expname in  ["EnsEcmwfSeas", "EnsIfmSeas","EnsCmccSeas", "EnsUkmoSeas"]):
            exp=DataEns(expname, varini)
        elif (expname in ["S4"]):
            print("bla")
            exp=DataS4(expname, varini)
        elif (expname in ["b0cm", "b0gq"]):
            exp=DataEC23(expname, varini)
        elif (expname in ["ERAint"]):
            d1_seas=date1+relativedelta(months=+1)
            d2_seas=date2+relativedelta(months=+1)
            listobs=list_season_obs(expname, varini, date1, date2, 4)
            exp=concat_monthly_data(listobs, varini)
        else:
            print("no type of exp assigned")
            sys.exit(0)

        exp.ready(date1, date2)
        ######seasonal values#####


        leadtime1=relativedelta(months=+1)
        leadtime2=relativedelta(months=+4, days=-1)
        exp_season=exp.select_leadtime(leadtime1, leadtime2)
        
        #seasonal percentile       
        
        for per in perlst:
            dictcomp={10:"less", 90:"greater"}
            comp=dictcomp.get(per)
            exp_stat2=exp_season.seasonal_percentile(per)
            # change startdate of the file to original one for naming convention
            exp_stat2.set_sdates(exp.get_sdates())
            exp_stat2.write_output()
            exp_stat3=exp_season.seasonal_nbday(per, comp)
            exp_stat3.set_sdates(exp.get_sdates())
            exp_stat3.write_output()
            
    shutil.rmtree(workdir) 
        

def calc_mon_percent_nbdays(varini, perlst, explst, year1, year2, monstart, workdir):
    """ this function will calculate and write the output
    of the seasonal (between leadtime month 02 and 04)
    number of day and the percentile per for all years between year1
    and year2 for all exp in explst and all observation in obslist
    -var: the name of the input variable
    -explist: list of experiments or observations
    -year1: first years of the time period
    -year2: last years of the time period
    -monstrt: startmonth
    -workdir: path of the working directory
    """
    
    
    date1=datetime.date(year1,monstart,1)
    date2=datetime.date(year2,monstart,1)



    #loop on experiments
    for expname in explst: 
        if (not os.path.isdir(workdir)):
            os.mkdir(workdir)
        os.chdir(workdir)

        #creating the object according to the type of experiment/observations
        if (expname in  ["EnsEcmwfSeas", "EnsIfmSeas","EnsCmccSeas", "EnsUkmoSeas"]):
            exp=DataEns(expname, varini)
        elif (expname in ["S4"]):
            print("bla")
            exp=DataS4(expname, varini)
        elif (expname in ["b0cm", "b0gq"]):
            exp=DataEC23(expname, varini)
        elif (expname in ["ERAint"]):
            d1_seas=date1+relativedelta(months=+1)
            d2_seas=date2+relativedelta(months=+1)
            listobs=list_season_obs(expname, varini, date1, date2, 4)
            exp=concat_monthly_data(listobs, varini)
        exp.see()
        exp.ready(date1, date2)
        exp.see()


        for per in perlst:
            #monthly percentile
            exp_stat2=exp.monthly_percentile(per)
            if not(expname in ["ERAint"]):
                exp_stat2.write_output()
                exp_stat2.see()
        
            
        for per in perlst:
          dictcomp={10:"less", 90:"greater"}
          comp=dictcomp.get(per)
          exp_stat3=exp.monthly_nbday(per, comp)
          if not(expname in ["ERAint"]):
              exp_stat3.write_output()

        shutil.rmtree(workdir) 
  
