import sets
import numpy
from dateutil.relativedelta import *
import netCDF4 as nc
import copy
import sys
import time
import datetime
from dateutil.relativedelta import *
import glob
import cdo as cdop
import os
#from data_obs import DataObs



def quit_leadtime(varname):
    l=varname.split("_")
    varnew=[]
    for part in l:
        if (len(part)!=5) and (list(part)[2]!="-"):
            varnew.append(part)
    return "_".join(varnew)

def get_last_date(f):
    """ get the last date of a file 
     -f: input file"""
    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']

    lastdate=((cdo.showtimestamp(input=f)[0]).split())[-1]
    lastdate=time.strptime(lastdate,'%Y-%m-%dT%H:%M:%S')
    lastdate=datetime.date(lastdate.tm_year, lastdate.tm_mon, lastdate.tm_mday)
    return  lastdate

def get_freq(f):
    """ get the frequency date of a file based on 
    the 2 first time steps
    -f: input file"""
    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']
    #print(f)
    listdate=((cdo.showtimestamp(input=f)[0]).split())
    date1=listdate[0]
    #print(date1)
    date1=time.strptime(date1,'%Y-%m-%dT%H:%M:%S')
    if (len(listdate)==1):
        freq=relativedelta(months=+1)
    else:
        date2=listdate[1]
        date2=time.strptime(date2,'%Y-%m-%dT%H:%M:%S')
        freq=datetime.date(
            date1.tm_year, 
            date1.tm_mon, date1.tm_mday) - datetime.date(
            date2.tm_year, 
            date2.tm_mon, date2.tm_mday) + datetime.timedelta(
                0, 0, 0, 0, 0, date2.tm_hour) - datetime.timedelta(
                0, 0, 0, 0, 0, date1.tm_hour)
    return  freq


def diff_month(d1, d2):
    """ return the number of month
    between 2 dates
    -d1: input date1
    -d2: input date2"""
    
    return (d1.year - d2.year)*12 + d1.month - d2.month


def settaxis(dataname, varname, f, d, freq):
    """test if the calendar is well written and return a another file 
    with the rigth time axis if needed
    - dataname: the name the experience
    - f: the file to be checked
    - d: the startdate of the file
    - freq: the frequency of the file
    """
    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']

    if freq == -1:
        freq=get_freq(f)
        if get_freq(f) == 0:
            print "no default value of frequency has been set for the %s experiment" % dataname
            print "and the calendar of the file is not readable"
            sys.exit(0)
    fileout="taxis_"+(f.split("/"))[-1]
    if freq.days >= 1:
        strfreq="%iday" % freq.days
    else:
        strfreq="%ihour" % (freq.seconds/3600)
    cdo.settaxis(d, "00:00", strfreq,  
                 input=f, output=fileout, options="-r -f nc")
    print "It has been rewriten with a %s frequency" % (strfreq)
    print fileout
    return fileout

def concat_monthly_data(listmondata, varout):
    """ return a concatenation of a list of 
    Data object, which contain monthly output
    with one timestep
    the concatenation is done between the first
    (and then second, third...) file
    of the filelist of all object of the list
    -listmondata: list of Data containing monthly files
    -varout: name of the output varible
    -expout: a data object containing the 
    temporal conatenation of the files of listmondata"""
    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']

    nbdata=len(listmondata)

    data0=listmondata[0]
    filelist0=data0.get_filelist()
    sdates0=data0.get_sdates()
    freq0=data0.get_freq()
    varname=data0.get_varname()

    nbfile=len(filelist0)
    if len(sdates0) != nbfile:
        print "the filelist and the sdates does not have the same length"
        sys.exit(0)

    filelistmon=[]
    sdatesmon=[]
    for data in listmondata:
        filelist=data.get_filelist()
        sdate=data.get_sdates()
        freq=data.get_freq()
        if len(filelist) != nbfile:
            print "all data does not have the same number of file"
            sys.exit(0)
        if len(sdate) != nbfile:
            print "the filelist and the sdates does not have the same length"
            sys.exit(0)
        if freq != freq0:
            print "all file does not have the same frequence"
            sys.exit(0)

    

    fileoutlist=[]
    for ifile in range(nbfile):
        #construct the list of file for 1 month
        filelistmon=[]
        sdatesmon=[]
        for data in listmondata:
            filelistmon.append((data.get_filelist())[ifile])
            sdatesmon.append((data.get_sdates())[ifile])

        for idata in range(nbdata-1):
            if sdatesmon[idata +1] != sdatesmon[idata] + relativedelta(months=+1):
                print "the file are not separated by one month"
                sys.exit(1)
        fileout=varout + "_" + str(sdatesmon[0]).replace("-","") + ".nc"
        cdo.mergetime(input=filelistmon, output=fileout, options= "-O -r -f nc -b 32")
        fileoutlist.append(fileout)

    if len(fileoutlist) != nbfile:
        print "the output list has %i instead of %i" % (len(fileoutlist),nbfile)
        sys.exit(1)
    expout=data0.duplicate()
    expout.set_varname(varout)
    expout.set_filelist(fileoutlist)
    expout.set_sdates(list(sdates0))
    return expout


def write_time(ncdfout, ncdfin):
    """fonction to write the time axis of a netcdf
    with simple time axis, should be extended to 
    double time axis
    input:
        -ncdfout: output netcdf open in write mode
        -ncdfin: Input netcdf in read mode
        """
    
    variables=ncdfin.variables.keys()
#read value from input file
    if "reftime" in ncdfin.variables.keys():
        timevar="reftime"
    elif "time" in ncdfin.variables.keys():
        timevar="time"
    else:
        print "no time variable found"
        sys.exit(0)
    reftimein = ncdfin.variables[timevar]
    
    #create the time dimension
    ncdfout.createDimension('time', None)

    #create the reftime variable
    reftime=ncdfout.createVariable(timevar, "f8", ('time',))
    reftime[:] = reftimein
    reftime.units = str(reftimein.units)
    reftime.standard_name = "forecast_reference_time"
    reftime.longname = "forecast reference time"
    reftime.calendar=str(reftimein.calendar)
    #print(reftimein.calendar)
    #sys.exit(0)
    #create the leadtime variable
    #leadtime=ncdfout.createVariable("leadtime", "i", ('time',))
    #to be changed?
    #leadtime[:] = numpy.array([20209,]*ntimestep)
    #print type(numpy.array([20209,]*ntimestep))
    #leadtime.units = "hours" 
    #leadtime.standard_name = "forecast_period"
    #leadtime.longname = "Time elapsed since the start of the forecast"
    #leadtime.bounds = "time_bnd"


def write_lonlat(ncdfout, ncdfin):
    variables=ncdfin.variables.keys()

            #read value from input file
    if 'longitude' in variables:
        lonin = ncdfin.variables['longitude']
    elif 'lon' in variables:
        lonin = ncdfin.variables['lon']
    else:
        print "longitude dimension is not in the file "
        sys.exit(0)
    if 'latitude' in variables:
        latin = ncdfin.variables['latitude']
    elif 'lat' in variables:
        latin = ncdfin.variables['lat']
    else:
        print "latitude dimension is not in the file "
        sys.exit(0)

    #create the spatial dimension
    ncdfout.createDimension('longitude', numpy.shape(lonin)[0])
    ncdfout.createDimension('latitude', numpy.shape(latin)[0])

    #create the longitude variable
    lon=ncdfout.createVariable("longitude", "f4", ('longitude',))
    lat=ncdfout.createVariable("latitude", "f4", ('latitude',))
    lon.data_type = "float"
    lat.data_type = "float"
    lon.units = "degrees_east"
    lat.units = "degrees_north"
    lon.axis = "X"
    lat.axis = "Y"
    lon.standard_name = "longitude"
    lat.standard_name = "latitude"
    lon.topology = "circular" 
    lon.modulo = 360
    lon.valid_min = 0.
    lat.valid_min = -89.
    lon.valid_max = 359.
    lat.valid_max = 89. 
    lon[:]=lonin
    lat[:]=latin


def write_insti(ncdfout,nmember,dataname):
    ncdfout.createDimension('string15', 15)
    dictinsti = { "EnsCmccSeas":"CMCC-Bologna   ",
                  "EnsIfmSeas":"IFM-GEOMAR     ",
                  "EnsMetfrSeas":"Meteo-France   ",
                  "EnsUkmoSeas":"UK Met Office  " , 
                  "EnsEcmwfSeas":"ECMWF          "
                  } 

    if dataname in dictinsti.keys():
        insti=dictinsti.get(dataname)
    else:
        print "The institution is not define"
        print "The Institution is set by default to IC3"
        insti="IC3            "
    institution=ncdfout.createVariable("institution", 'c', ('ensemble','string15',))
    institution.standard_name = "institution"
    institution.long_name = "Institution responsible for the forecast system"
    institution[:] = numpy.array([numpy.array(list(insti)),]*nmember)

def write_id(ncdfout,nmember,dataname):
    ncdfout.createDimension("string4", 4)
    dictid = { "EnsCmccSeas":"2001",
               "EnsIfmSeas":"2001",
               "EnsMetfrSeas":"2001",
               "EnsUkmoSeas":"2001" , 
               "EnsEcmwfSeas":"2001" ,
               "S4":"s004" } 
    if dataname in dictid.keys():
        id=dictid.get(dataname)
    else:
        id=dataname
        
    experiment_id=ncdfout.createVariable("experiment_id", 'c', ('ensemble','string4',))
    experiment_id.standard_name = "experiment_id"
    experiment_id.long_name = "Experiment identifier" 
    experiment_id[:]=numpy.array([numpy.array(list(id)),]*nmember)


def write_realization(ncdfout,nmember):
    realization=ncdfout.createVariable("realization", 'c', ('ensemble',))
    realization.standard_name = "realization"
    realization.long_name = "Number of the simulation in the ensemble" 
    realization[:] = numpy.arange(nmember)


def write_source(ncdfout,nmember,dataname):
    ncdfout.createDimension('string60', 60)
    dictsource = { "EnsCmccSeas":"ECHAM5/OPA8.2, Sys 1, Met 0, ENSEMBLES                      ",
                   "EnsIfmSeas":"ECHAM5 T63L31/MPIOM GR15L40, Sys 1, Met 10, ENSEMBLES       ",
                   "EnsMetfrSeas":"ARPEGEClimate4.6/OPA8.2/GELATO, Sys 0, Met 1, ENSEMBLES     ",
                   "EnsUkmoSeas":"HadGEM2, Sys 1, Met 1, ENSEMBLES                            ", 
                   "EnsEcmwfSeas":"IFS31R1/HOPE-E, Sys 1, Met 1, ENSEMBLES                     ",
                   "b0cm":"EC-Earth2.3.0,NEMOVAR-COMBINE,DFS4.3,ERA40/Int,Atm_SV+O pert",
                   "b0gq":"EC-Earth2.3.0,NEMOVAR-COMBINE,DFS4.3,ERA40/Int,Atm_SV+O pert",
                   "S4":"Sys 004, Met 001, Operational                               "} 
    
    if dataname in dictsource.keys():
        src=dictsource.get(dataname)
    else:
        print "The institution is not define"
        print "There is no default value for source"
        sys.exit(0)
        
    source=ncdfout.createVariable("source", 'c', ('ensemble','string60',))
    source.standard_name = "source"
    source.long_name = "Method of production of the data"
    source[:] = numpy.array([numpy.array(list(src)),]*nmember)
        
def weighted_sum(listdata, listcoeff, varout):
    """Create the ponderate mean between 
    the files of listdata
    listdata: list of data object on which is done the mean
    listcoeff: coefficient to be apply on the different files"""

    
    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']

    nbdata=len(listdata)

    data0=listdata[0]
    filelist0=data0.get_filelist()
    sdates0=data0.get_sdates()
    freq0=data0.get_freq()
    varname0=data0.get_varname()

    nbfile=len(filelist0)
    if len(sdates0) != nbfile:
        print "the filelist and the sdates does not have the same length"
        sys.exit(0)

    filelist=[]
    sdates=[]
    for data in listdata:
        filelist=data.get_filelist()
        sdates=data.get_sdates()
        freq=data.get_freq()
        if len(filelist) != nbfile:
            print "all data does not have the same number of file"
            sys.exit(0)
        if len(sdates) != nbfile:
            print "the filelist and the sdates does not have the same length"
            sys.exit(0)
        if freq != freq0:
            print "all file does not have the same frequence"
            sys.exit(0)

    

    fileoutlist=[]
    for ifile in range(nbfile):
        #construct the list of file for 1 th
        f=filelist0[ifile]
        #keep the old name of the file to handle AtlCMIP class and avoid the overwritting of the files
        fileout=varout + "_" + str(sdates0[ifile]).replace("-","") + f #".nc"
        f0=cdo.mulc(listcoeff[0], input=f)

        for idata in range(1,len(listdata)):
            f=(data.get_filelist())[ifile]
            fsum=cdo.chvar(varname0,varout, input=" -add -mulc,"+str(listcoeff[idata])+" "+f0+" "+f, 
                           output=fileout)

        fileoutlist.append(fileout)  

    if len(fileoutlist) != nbfile:
        print "the output list has %i instead of %i" % (len(fileoutlist),nbfile)
        sys.exit(0)
    expout=data0.duplicate()
    expout.set_varname(varout)
    expout.set_filelist(fileoutlist)
    expout.set_sdates(sdates0)
    return expout


def compute_reg(filein, dataname, regname, varname, fldmean=True):
    """this function select a give region
    with or without masking the land/sea
    arguments:
    - filein: input file
    - dataname: name of the data 
    (to choose the resolution for the mask)
    - name of the region 
    (the correponding lon lat has to defined 
    in the function)
    outputs:
    - netcdf file name with the computed index
    """

    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']

    fileaux=filein.split("/")[-1]
    fileout=regname+"_"+fileaux

    regdict={"ISMR":(70,95,5,30),
             "France":(360-5,5,44,50),
             "soil2003":(-5,20,43,55),
             "soil2010":(25,55,45,60),
             "ATL3":(-20,0,-3,3),
             "CentralAtl":(-33,0,-3,3),
             "AtlBias":(-20,10,-14,3),
             "Beng":(0, 15, -14, -3),
             "Nino34":(190,240,-5,5),
             "BENGUELA":(12,20,-32,-20),
             "ANGOLA":(10,20,-15,-3),
             "SOUTHEAST":(0,10,-20,-5),
             "Europe":(-20,70,25,75),           
             "EuropeSmall":(-20,45,35,70),           
             "EuropeNorth":(-20,45,45,70),           
             "EuropeSouth":(-20,45,35,45),           
             "NAE":(-80,40,20,80)  
         }
    maskreg={"ISMR":"land",
             "France":"land",
             "soil2003":"land",
             "soil2010":"land",
             "ATL3":"oce",
             "CentralAtl":"oce",
             "AtlBias":"oce",
             "Beng":"oce",
             "BENGUELA":"oce",
             "Nino34":"oce",
             "ANGOLA":"oce",
             "SOUTHEAST":"oce",
             "Europe":"land",
             "EuropeSmall":"land",
             "EuropeSouth":"land",
             "EuropeNorth":"land",
             "NAE":""  
             }
    griddict={"m02j":"r512x256",
              "m02r":"r512x256",
              "m04r":"r512x256",
              "m04o":"r512x256",
              "S4":"r360x181",
              "a01d":"r512x256",
              "a01c":"r512x256",
              "m02s":"r1024x512",
              "m04s":"r1024x512",
              "m04p":"r1024x512",
              "ERAint":"r480x241",
              "ERAland":"r512x256",
              "b0cm":"r320x160",
              "b0gq":"r320x160",
              "AtlCMIP5":"alreadymasked"
                  }
    reg=regdict.get(regname)
    if not(reg):
        print("the region %s is not defined"%reg)
        sys.exit(1)
    masktype=maskreg.get(regname)
    grid=griddict.get(dataname)
    if grid=="alreadymasked":
        fmask=cdo.gtc(100000,input=filein)
    else:
        fmask="/esnas/exp/ecearth/"+grid.replace("r","land_sea_mask_")+".nc"
 
    maskop="div"
    #print(fmask)
    if masktype == "land":
        fmask=cdo.setctomiss(0,input="-gtc,0.5 "+fmask)
    elif masktype == "oce":
        fmask=cdo.setctomiss(0,input="-ltc,0.5 "+fmask)
    elif masktype == "":
        #create a mask full of 1
        fmask=cdo.addc(1,input="-mulc,0 "+fmask)

    #create a mask with the correct number of level
    #count the number of level
    nmember=int(cdo.nlevel(input=filein)[0])
    lstmask=[fmask]*nmember
    fmask=cdo.merge(input=lstmask)
    regstr=','.join(str(x) for x in reg)
    #average or not the field over the region

    if fldmean:
        fldmeanstr="-fldmean"
    else:
        fldmeanstr=""
    if (len(glob.glob(fileout))==0):
        #if grid=="alreadymasked":
        cdo.chvar(varname, varname+"_"+regname, 
            input=fldmeanstr+" -sellonlatbox,"+regstr+" -"+maskop+" "+filein+" "+fmask,output=fileout)
        #else:
            #probel already occured and solved with " -setgrid,"+ grid but seems to create issue in other case
            #for now remove it...
        #    cdo.chvar(varname, varname+"_"+regname, 
        #              input=fldmeanstr+" -sellonlatbox,"+regstr+" -"+maskop+" "+filein+" "+fmask,
        #              output=fileout)

    return(fileout)


def calc_fst_thres(filein,thres, varname):
    """calculate the frirst day exceeding a threshold 
    from a file
    - filein: the precipitation file
    - thres: the threshold
    - var: the variable name inside the file
    """

    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']


    f=filein #cdo.runmean(3,input=filein)
    #print(f)
    ncdfin=nc.Dataset(f, "r")
    #print(varname)
    ISMR=ncdfin.variables[varname]
    shapin=(numpy.array(ISMR[:])).shape
    ISMR=numpy.resize(ISMR,(shapin[0],shapin[1]))
    if shapin[1]==1:
        onset=numpy.where(ISMR>thres)[0][0]
    else:
        onset=numpy.array([numpy.where(row>thres)[0][0] 
                           for s,row in enumerate(numpy.transpose(ISMR))])
    #print(onset)
    return(onset)



       
def module(data1, data2, varout):
    """Calculated the module of 2 data (wind or windstress...)
    data1 and data2
    data1: first data
    data2: second data
    varout: name of the output variable"""
    
    cdo=cdop.Cdo()
    cdo.tempfile.tempdir = os.environ['TMPDIR']

    nbdata=2

    filelist1=data1.get_filelist()
    sdates1=data1.get_sdates()
    freq1=data1.get_freq()
    varname1=data1.get_varname()
    varname2=data2.get_varname()

    nbfile=len(filelist1)
    if len(sdates1) != nbfile:
        print "the filelist and the sdates does not have the same length"
        sys.exit(0)

    filelist=[]
    sdates=[]
    filelist2=data2.get_filelist()
    sdates2=data2.get_sdates()
    freq2=data2.get_freq()
    if len(filelist2) != nbfile:
        print "all data does not have the same number of file"
        sys.exit(0)
    if len(sdates2) != nbfile:
        print "the filelist and the sdates does not have the same length"
        sys.exit(0)
    if freq2 != freq1:
        print "all file does not have the same frequence"
        sys.exit(0)

    fileoutlist=[]
    for ifile in range(nbfile):
        #construct the list of file for 1 th
        fileout=varout + "_" + str(sdates1[ifile]).replace("-","") + ".nc"
        f1=filelist1[ifile]
        f2=filelist2[ifile]

        cdo.chvar(varname1,varout, 
                  input="-sqrt -add -mul  "+f1+" "+f1 + " -mul  "+f2+" "+f2 , 
                  output=fileout)

        fileoutlist.append(fileout)  

    if len(fileoutlist) != nbfile:
        print "the output list has %i instead of %i" % (len(fileoutlist),nbfile)
        sys.exit(0)
    expout=data1.duplicate()
    expout.set_varname(varout)
    expout.set_filelist(fileoutlist)
    expout.set_sdates(sdates1)
    return expout

